from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class NestedResourceClassMethodsTests(StripeTestCase):
    @stripe.api_resources.abstract.nested_resource_class_methods(
        'nested',
        operations=['create', 'retrieve', 'update', 'delete', 'list']
    )
    class MainResource(stripe.api_resources.abstract.APIResource):
        pass

    def test_create_nested(self):
        self.stub_request(
            'post',
            '/v1/mainresources/id/nesteds',
            {
                'id': 'nested_id',
                'object': 'nested',
                'foo': 'bar',
            }
        )
        nested_resource = self.MainResource.create_nested('id', foo='bar')
        self.assert_requested(
            'post',
            '/v1/mainresources/id/nesteds',
            {
                'foo': 'bar'
            },
            None
        )
        self.assertEqual('bar', nested_resource.foo)

    def test_retrieve_nested(self):
        self.stub_request(
            'get',
            '/v1/mainresources/id/nesteds/nested_id',
            {
                'id': 'nested_id',
                'object': 'nested',
                'foo': 'bar',
            }
        )
        nested_resource = self.MainResource.retrieve_nested('id', 'nested_id')
        self.assert_requested(
            'get',
            '/v1/mainresources/id/nesteds/nested_id',
            {},
            None
        )
        self.assertEqual('bar', nested_resource.foo)

    def test_modify_nested(self):
        self.stub_request(
            'post',
            '/v1/mainresources/id/nesteds/nested_id',
            {
                'id': 'nested_id',
                'object': 'nested',
                'foo': 'baz',
            }
        )
        nested_resource = self.MainResource.modify_nested('id', 'nested_id',
                                                          foo='baz')
        self.assert_requested(
            'post',
            '/v1/mainresources/id/nesteds/nested_id',
            {
                'foo': 'baz'
            },
            None
        )
        self.assertEqual('baz', nested_resource.foo)

    def test_delete_nested(self):
        self.stub_request(
            'delete',
            '/v1/mainresources/id/nesteds/nested_id',
            {
                'id': 'nested_id',
                'object': 'nested',
                'deleted': True,
            }
        )
        nested_resource = self.MainResource.delete_nested('id', 'nested_id')
        self.assert_requested(
            'delete',
            '/v1/mainresources/id/nesteds/nested_id',
            {},
            None
        )
        self.assertEqual(True, nested_resource.deleted)

    def test_list_nesteds(self):
        self.stub_request(
            'get',
            '/v1/mainresources/id/nesteds',
            {
                'object': 'list',
                'data': [],
            }
        )
        nested_resource = self.MainResource.list_nesteds('id')
        self.assert_requested(
            'get',
            '/v1/mainresources/id/nesteds',
            {},
            None
        )
        self.assertTrue(isinstance(nested_resource.data, list))



from __future__ import absolute_import, division, print_function

import os
import sys
import unittest2

from distutils.version import StrictVersion

import stripe
from stripe.six.moves.urllib.request import urlopen
from stripe.six.moves.urllib.error import HTTPError

from tests.request_mock import RequestMock


MOCK_MINIMUM_VERSION = '0.16.1'
MOCK_PORT = os.environ.get('STRIPE_MOCK_PORT', 12111)


try:
    resp = urlopen('http://localhost:%s/' % MOCK_PORT)
    info = resp.info()
except HTTPError as e:
    info = e.info()
except Exception:
    sys.exit("Couldn't reach stripe-mock at `localhost:%s`. Is "
             "it running? Please see README for setup instructions." %
             MOCK_PORT)

version = info.get('Stripe-Mock-Version')
if version != 'master' \
        and StrictVersion(version) < StrictVersion(MOCK_MINIMUM_VERSION):
    sys.exit("Your version of stripe-mock (%s) is too old. The minimum "
             "version to run this test suite is %s. Please "
             "see its repository for upgrade instructions." %
             (version, MOCK_MINIMUM_VERSION))


class StripeTestCase(unittest2.TestCase):
    def setUp(self):
        super(StripeTestCase, self).setUp()

        stripe.orig_attrs = {
            'api_base': stripe.api_base,
            'api_key': stripe.api_key,
            'client_id': stripe.client_id,
            'default_http_client': stripe.default_http_client,
        }
        stripe.api_base = 'http://localhost:%s' % MOCK_PORT
        stripe.api_key = 'sk_test_123'
        stripe.client_id = 'ca_123'

        self.client = stripe.http_client.new_default_http_client()
        stripe.default_http_client = self.client

        self.request_mock = RequestMock()
        self.request_mock.start()

    def tearDown(self):
        super(StripeTestCase, self).tearDown()

        self.request_mock.stop()

        self.client.close()

        stripe.api_base = stripe.orig_attrs['api_base']
        stripe.api_key = stripe.orig_attrs['api_key']
        stripe.client_id = stripe.orig_attrs['client_id']
        stripe.default_http_client = stripe.orig_attrs['default_http_client']

    def stub_request(self, *args, **kwargs):
        return self.request_mock.stub_request(*args, **kwargs)

    def assert_requested(self, *args, **kwargs):
        return self.request_mock.assert_requested(*args, **kwargs)

    def assert_no_request(self):
        return self.request_mock.assert_no_request()



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'trr_123'


class ReversalTest(StripeTestCase):
    def construct_resource(self):
        reversal_dict = {
            'id': TEST_RESOURCE_ID,
            'object': 'reversal',
            'metadata': {},
            'transfer': 'tr_123'
        }
        return stripe.Reversal.construct_from(reversal_dict, stripe.api_key)

    def test_has_instance_url(self):
        resource = self.construct_resource()
        self.assertEqual(
            '/v1/transfers/tr_123/reversals/%s' % TEST_RESOURCE_ID,
            resource.instance_url()
        )

    def test_is_not_modifiable(self):
        with self.assertRaises(NotImplementedError):
            stripe.Reversal.modify(
                TEST_RESOURCE_ID,
                metadata={'key': 'value'}
            )

    def test_is_not_retrievable(self):
        with self.assertRaises(NotImplementedError):
            stripe.Reversal.retrieve(TEST_RESOURCE_ID)

    # We don't use stripe-mock as the reversal returned has a transfer id that
    # is different from the transfer used to access the reversal
    def test_is_saveable(self):
        resource = self.construct_resource()
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/transfers/tr_123/reversals/%s' % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class ListObjectTests(StripeTestCase):

    def setUp(self):
        super(ListObjectTests, self).setUp()

        self.lo = stripe.ListObject.construct_from({
            'id': 'me',
            'url': '/my/path',
            'data': ['foo'],
        }, 'mykey')

    def assertResponse(self, res):
        self.assertTrue(isinstance(res[0], stripe.Charge))
        self.assertEqual('bar', res[0].foo)

    def test_for_loop(self):
        seen = []

        for item in self.lo:
            seen.append(item)

        self.assertEqual(['foo'], seen)

    def test_list(self):
        self.stub_request(
            'get',
            '/my/path',
            [
                {
                    'object': 'charge',
                    'foo': 'bar',
                },
            ]
        )

        res = self.lo.list(myparam='you')

        self.assert_requested(
            'get',
            '/my/path',
            {
                'myparam': 'you',
            },
            None
        )
        self.assertResponse(res)

    def test_create(self):
        self.stub_request(
            'post',
            '/my/path',
            [
                {
                    'object': 'charge',
                    'foo': 'bar',
                },
            ]
        )

        res = self.lo.create(myparam='eter')

        self.assert_requested(
            'post',
            '/my/path',
            {
                'myparam': 'eter',
            },
            None
        )
        self.assertResponse(res)

    def test_retrieve(self):
        self.stub_request(
            'get',
            '/my/path/myid',
            [
                {
                    'object': 'charge',
                    'foo': 'bar',
                },
            ]
        )

        res = self.lo.retrieve('myid', myparam='cow')

        self.assert_requested(
            'get',
            '/my/path/myid',
            {
                'myparam': 'cow',
            },
            None
        )

        self.assertResponse(res)

    def test_len(self):
        self.assertEqual(len(self.lo), 1)

    def test_bool(self):
        self.assertTrue(self.lo)

        empty = stripe.ListObject.construct_from({
            'id': 'me',
            'url': '/my/path',
            'data': [],
        }, 'mykey')
        self.assertFalse(empty)


class AutoPagingTests(StripeTestCase):

    @staticmethod
    def pageable_model_response(ids, has_more):
        return {
            'object': 'list',
            'url': '/v1/pageablemodels',
            'data': [{'id': id, 'object': 'pageablemodel'} for id in ids],
            'has_more': has_more,
        }

    def test_iter_one_page(self):
        lo = stripe.ListObject.construct_from(
            self.pageable_model_response(['pm_123', 'pm_124'], False),
            'mykey'
        )

        self.assert_no_request()

        seen = [item['id'] for item in lo.auto_paging_iter()]

        self.assertEqual(['pm_123', 'pm_124'], seen)

    def test_iter_two_pages(self):
        lo = stripe.ListObject.construct_from(
            self.pageable_model_response(['pm_123', 'pm_124'], True),
            'mykey'
        )
        lo._retrieve_params = {'foo': 'bar'}

        self.stub_request(
            'get',
            '/v1/pageablemodels',
            self.pageable_model_response(['pm_125', 'pm_126'], False)
        )

        seen = [item['id'] for item in lo.auto_paging_iter()]

        self.assert_requested(
            'get',
            '/v1/pageablemodels',
            {
                'starting_after': 'pm_124',
                'foo': 'bar'
            },
            None
        )

        self.assertEqual(['pm_123', 'pm_124', 'pm_125', 'pm_126'], seen)

    def test_class_method_two_pages(self):
        self.stub_request(
            'get',
            '/v1/charges',
            {
                'object': 'list',
                'data': [{'id': 'ch_001'}],
                'url': '/v1/charges',
                'has_more': False,
            }
        )

        seen = [item['id'] for item in stripe.Charge.auto_paging_iter(
            limit=25,
            foo='bar'
        )]

        self.assert_requested(
            'get',
            '/v1/charges',
            {
                'limit': 25,
                'foo': 'bar',
            }
        )
        self.assertEqual(['ch_001'], seen)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class MyDeletable(stripe.api_resources.abstract.DeletableAPIResource):
    pass


class DeletableAPIResourceTests(StripeTestCase):
    def test_delete(self):
        self.stub_request(
            'delete',
            '/v1/mydeletables/mid',
            {
                'id': 'mid',
                'deleted': True,
            },
            rheaders={'request-id': 'req_id'}
        )

        obj = MyDeletable.construct_from({
            'id': 'mid'
        }, 'mykey')

        self.assertTrue(obj is obj.delete())
        self.assert_requested(
            'delete',
            '/v1/mydeletables/mid',
            {}
        )
        self.assertEqual(True, obj.deleted)
        self.assertEqual('mid', obj.id)

        self.assertTrue(obj.last_response is not None)
        self.assertEqual('req_id', obj.last_response.request_id)



import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'issfr_123'


class IssuerFraudRecordTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.IssuerFraudRecord.list()
        self.assert_requested(
            'get',
            '/v1/issuer_fraud_records'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.IssuerFraudRecord)

    def test_is_retrievable(self):
        resource = stripe.IssuerFraudRecord.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/issuer_fraud_records/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.IssuerFraudRecord)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class MyListable(stripe.api_resources.abstract.ListableAPIResource):
    pass


class ListableAPIResourceTests(StripeTestCase):

    def test_all(self):
        self.stub_request(
            'get',
            '/v1/mylistables',
            {
                'object': 'list',
                'data': [
                    {
                        'object': 'charge',
                        'name': 'jose',
                    },
                    {
                        'object': 'charge',
                        'name': 'curly',
                    }
                ],
                'url': '/v1/charges',
                'has_more': False,
            },
            rheaders={'request-id': 'req_id'}
        )

        res = MyListable.list()
        self.assert_requested(
            'get',
            '/v1/mylistables',
            {}
        )
        self.assertEqual(2, len(res.data))
        self.assertTrue(all(isinstance(obj, stripe.Charge)
                            for obj in res.data))
        self.assertEqual('jose', res.data[0].name)
        self.assertEqual('curly', res.data[1].name)

        self.assertTrue(res.last_response is not None)
        self.assertEqual('req_id', res.last_response.request_id)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class BalanceTest(StripeTestCase):
    def test_is_retrievable(self):
        resource = stripe.Balance.retrieve()
        self.assert_requested(
            'get',
            '/v1/balance'
        )
        self.assertIsInstance(resource, stripe.Balance)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class MySingleton(stripe.api_resources.abstract.SingletonAPIResource):
    pass


class SingletonAPIResourceTests(StripeTestCase):

    def test_retrieve(self):
        self.stub_request(
            'get',
            '/v1/mysingleton',
            {
                'single': 'ton'
            },
            rheaders={'request-id': 'req_id'}
        )

        res = MySingleton.retrieve()

        self.assert_requested(
            'get',
            '/v1/mysingleton',
            {}
        )
        self.assertEqual('ton', res.single)
        self.assertTrue(res.last_response is not None)
        self.assertEqual('req_id', res.last_response.request_id)



from __future__ import absolute_import, division, print_function

import unittest2
from mock import MagicMock, Mock, patch

import stripe
from stripe import six

VALID_API_METHODS = ('get', 'post', 'delete')


class StripeClientTestCase(unittest2.TestCase):
    REQUEST_LIBRARIES = ['urlfetch', 'requests', 'pycurl', 'urllib.request']

    def setUp(self):
        super(StripeClientTestCase, self).setUp()

        self.request_patchers = {}
        self.request_mocks = {}
        for lib in self.REQUEST_LIBRARIES:
            patcher = patch("stripe.http_client.%s" % (lib,))

            self.request_mocks[lib] = patcher.start()
            self.request_patchers[lib] = patcher

    def tearDown(self):
        super(StripeClientTestCase, self).tearDown()

        for patcher in six.itervalues(self.request_patchers):
            patcher.stop()


class HttpClientTests(StripeClientTestCase):

    def setUp(self):
        super(HttpClientTests, self).setUp()

        self.original_filters = stripe.http_client.warnings.filters[:]
        stripe.http_client.warnings.simplefilter('ignore')

    def tearDown(self):
        stripe.http_client.warnings.filters = self.original_filters

        super(HttpClientTests, self).tearDown()

    def check_default(self, none_libs, expected):
        for lib in none_libs:
            setattr(stripe.http_client, lib, None)

        inst = stripe.http_client.new_default_http_client()

        self.assertTrue(isinstance(inst, expected))

    def test_new_default_http_client_urlfetch(self):
        self.check_default((),
                           stripe.http_client.UrlFetchClient)

    def test_new_default_http_client_requests(self):
        self.check_default(('urlfetch',),
                           stripe.http_client.RequestsClient)

    def test_new_default_http_client_pycurl(self):
        self.check_default(('urlfetch', 'requests',),
                           stripe.http_client.PycurlClient)

    def test_new_default_http_client_urllib2(self):
        self.check_default(('urlfetch', 'requests', 'pycurl'),
                           stripe.http_client.Urllib2Client)


class ClientTestBase():

    @property
    def request_mock(self):
        return self.request_mocks[self.request_client.name]

    @property
    def valid_url(self, path='/foo'):
        return 'https://api.stripe.com%s' % (path,)

    def make_request(self, method, url, headers, post_data):
        client = self.request_client(verify_ssl_certs=True)
        return client.request(method, url, headers, post_data)

    def mock_response(self, body, code):
        raise NotImplementedError(
            'You must implement this in your test subclass')

    def mock_error(self, error):
        raise NotImplementedError(
            'You must implement this in your test subclass')

    def check_call(self, meth, abs_url, headers, params):
        raise NotImplementedError(
            'You must implement this in your test subclass')

    def test_request(self):
        self.mock_response(self.request_mock, '{"foo": "baz"}', 200)

        for meth in VALID_API_METHODS:
            abs_url = self.valid_url
            data = ''

            if meth != 'post':
                abs_url = '%s?%s' % (abs_url, data)
                data = None

            headers = {'my-header': 'header val'}

            body, code, _ = self.make_request(
                meth, abs_url, headers, data)

            self.assertEqual(200, code)
            self.assertEqual('{"foo": "baz"}', body)

            self.check_call(self.request_mock, meth, abs_url,
                            data, headers)

    def test_exception(self):
        self.mock_error(self.request_mock)
        self.assertRaises(stripe.error.APIConnectionError,
                          self.make_request,
                          'get', self.valid_url, {}, None)


class RequestsVerify(object):

    def __eq__(self, other):
        return other and other.endswith('stripe/data/ca-certificates.crt')


class RequestsClientTests(StripeClientTestCase, ClientTestBase):
    request_client = stripe.http_client.RequestsClient

    def setUp(self):
        super(RequestsClientTests, self).setUp()
        self.session = MagicMock()

    def test_timeout(self):
        headers = {'my-header': 'header val'}
        data = ''
        self.mock_response(self.request_mock, '{"foo": "baz"}', 200)
        self.make_request('POST', self.valid_url,
                          headers, data, timeout=5)

        self.check_call(None, 'POST', self.valid_url,
                        data, headers, timeout=5)

    def make_request(self, method, url, headers, post_data, timeout=80):
        client = self.request_client(verify_ssl_certs=True,
                                     timeout=timeout,
                                     proxy='http://slap/')

        return client.request(method, url, headers, post_data)

    def mock_response(self, mock, body, code):
        result = Mock()
        result.content = body
        result.status_code = code

        self.session.request = MagicMock(return_value=result)
        mock.Session = MagicMock(return_value=self.session)

    def mock_error(self, mock):
        mock.exceptions.RequestException = Exception
        self.session.request.side_effect = mock.exceptions.RequestException()
        mock.Session = MagicMock(return_value=self.session)

    # Note that unlike other modules, we don't use the "mock" argument here
    # because we need to run the request call against the internal mock
    # session.
    def check_call(self, mock, meth, url, post_data, headers, timeout=80):
        self.session.request. \
            assert_called_with(meth, url,
                               headers=headers,
                               data=post_data,
                               verify=RequestsVerify(),
                               proxies={"http": "http://slap/",
                                        "https": "http://slap/"},
                               timeout=timeout)


class UrlFetchClientTests(StripeClientTestCase, ClientTestBase):
    request_client = stripe.http_client.UrlFetchClient

    def mock_response(self, mock, body, code):
        result = Mock()
        result.content = body
        result.status_code = code

        mock.fetch = Mock(return_value=result)

    def mock_error(self, mock):
        mock.Error = mock.InvalidURLError = Exception
        mock.fetch.side_effect = mock.InvalidURLError()

    def check_call(self, mock, meth, url, post_data, headers):
        mock.fetch.assert_called_with(
            url=url,
            method=meth,
            headers=headers,
            validate_certificate=True,
            deadline=55,
            payload=post_data
        )


class Urllib2ClientTests(StripeClientTestCase, ClientTestBase):
    request_client = stripe.http_client.Urllib2Client

    def make_request(self, method, url, headers, post_data, proxy=None):
        self.client = self.request_client(verify_ssl_certs=True,
                                          proxy=proxy)
        self.proxy = proxy
        return self.client.request(method, url, headers, post_data)

    def mock_response(self, mock, body, code):
        response = Mock
        response.read = Mock(return_value=body)
        response.code = code
        response.info = Mock(return_value={})

        self.request_object = Mock()
        mock.Request = Mock(return_value=self.request_object)

        mock.urlopen = Mock(return_value=response)

        opener = Mock
        opener.open = Mock(return_value=response)
        mock.build_opener = Mock(return_value=opener)
        mock.build_opener.open = opener.open
        mock.ProxyHandler = Mock(return_value=opener)

        mock.urlopen = Mock(return_value=response)

    def mock_error(self, mock):
        mock.urlopen.side_effect = ValueError
        mock.build_opener().open.side_effect = ValueError
        mock.build_opener.reset_mock()

    def check_call(self, mock, meth, url, post_data, headers):
        if six.PY3 and isinstance(post_data, six.string_types):
            post_data = post_data.encode('utf-8')

        mock.Request.assert_called_with(url, post_data, headers)

        if (self.client._proxy):
            self.assertTrue(type(self.client._proxy) is dict)
            mock.ProxyHandler.assert_called_with(self.client._proxy)
            mock.build_opener.open.assert_called_with(self.request_object)
            self.assertTrue(not mock.urlopen.called)

        if (not self.client._proxy):
            mock.urlopen.assert_called_with(self.request_object)
            self.assertTrue(not mock.build_opener.called)
            self.assertTrue(not mock.build_opener.open.called)


class Urllib2ClientHttpsProxyTests(Urllib2ClientTests):
    def make_request(self, method, url, headers, post_data, proxy=None):
        return super(Urllib2ClientHttpsProxyTests, self).make_request(
                    method, url, headers, post_data,
                    {"http": "http://slap/",
                     "https": "http://slap/"})


class Urllib2ClientHttpProxyTests(Urllib2ClientTests):
    def make_request(self, method, url, headers, post_data, proxy=None):
        return super(Urllib2ClientHttpProxyTests, self).make_request(
                    method, url, headers, post_data,
                    "http://slap/")


class PycurlClientTests(StripeClientTestCase, ClientTestBase):
    request_client = stripe.http_client.PycurlClient

    def make_request(self, method, url, headers, post_data, proxy=None):
        self.client = self.request_client(verify_ssl_certs=True,
                                          proxy=proxy)
        self.proxy = proxy
        return self.client.request(method, url, headers, post_data)

    @property
    def request_mock(self):
        if not hasattr(self, 'curl_mock'):
            lib_mock = self.request_mocks[self.request_client.name]

            self.curl_mock = Mock()

            lib_mock.Curl = Mock(return_value=self.curl_mock)

        return self.curl_mock

    def setUp(self):
        super(PycurlClientTests, self).setUp()

        self.bio_patcher = patch('stripe.util.io.BytesIO')

        bio_mock = Mock()
        self.bio_patcher.start().return_value = bio_mock
        self.bio_getvalue = bio_mock.getvalue

    def tearDown(self):
        super(PycurlClientTests, self).tearDown()

        self.bio_patcher.stop()

    def mock_response(self, mock, body, code):
        self.bio_getvalue.return_value = body.encode('utf-8')

        mock.getinfo.return_value = code

    def mock_error(self, mock):
        class FakeException(BaseException):
            @property
            def args(self):
                return ('foo', 'bar')

        stripe.http_client.pycurl.error = FakeException
        mock.perform.side_effect = stripe.http_client.pycurl.error

    def check_call(self, mock, meth, url, post_data, headers):
        lib_mock = self.request_mocks[self.request_client.name]

        # A note on methodology here: we don't necessarily need to verify
        # _every_ call to setopt, but check a few of them to make sure the
        # right thing is happening. Keep an eye specifically on conditional
        # statements where things are more likely to go wrong.

        self.curl_mock.setopt.assert_any_call(lib_mock.NOSIGNAL, 1)
        self.curl_mock.setopt.assert_any_call(lib_mock.URL,
                                              stripe.util.utf8(url))

        if meth == 'get':
            self.curl_mock.setopt.assert_any_call(lib_mock.HTTPGET, 1)
        elif meth == 'post':
            self.curl_mock.setopt.assert_any_call(lib_mock.POST, 1)
        else:
            self.curl_mock.setopt.assert_any_call(lib_mock.CUSTOMREQUEST,
                                                  meth.upper())

        self.curl_mock.perform.assert_any_call()


class PycurlClientHttpProxyTests(PycurlClientTests):
    def make_request(self, method, url, headers, post_data, proxy=None):
        return super(PycurlClientHttpProxyTests, self).make_request(
                    method, url, headers, post_data,
                    "http://user:withPwd@slap:8888/")

    def check_call(self, mock, meth, url, post_data, headers):
        lib_mock = self.request_mocks[self.request_client.name]

        self.curl_mock.setopt.assert_any_call(lib_mock.PROXY, "slap")
        self.curl_mock.setopt.assert_any_call(lib_mock.PROXYPORT, 8888)
        self.curl_mock.setopt.assert_any_call(lib_mock.PROXYUSERPWD,
                                              "user:withPwd")

        super(PycurlClientHttpProxyTests, self).check_call(
                mock, meth, url, post_data, headers)


class PycurlClientHttpsProxyTests(PycurlClientTests):
    def make_request(self, method, url, headers, post_data, proxy=None):
        return super(PycurlClientHttpsProxyTests, self).make_request(
                    method, url, headers, post_data,
                    {"http": "http://slap:8888/",
                     "https": "http://slap2:444/"})

    def check_call(self, mock, meth, url, post_data, headers):
        lib_mock = self.request_mocks[self.request_client.name]

        self.curl_mock.setopt.assert_any_call(lib_mock.PROXY, "slap2")
        self.curl_mock.setopt.assert_any_call(lib_mock.PROXYPORT, 444)

        super(PycurlClientHttpsProxyTests, self).check_call(
                mock, meth, url, post_data, headers)


class APIEncodeTest(StripeClientTestCase):

    def test_encode_dict(self):
        body = {
            'foo': {
                'dob': {
                    'month': 1,
                },
                'name': 'bat'
            },
        }

        values = [t for t in stripe.api_requestor._api_encode(body)]

        self.assertTrue(('foo[dob][month]', 1) in values)
        self.assertTrue(('foo[name]', 'bat') in values)

    def test_encode_array(self):
        body = {
            'foo': [{
                'dob': {
                    'month': 1,
                },
                'name': 'bat'
            }],
        }

        values = [t for t in stripe.api_requestor._api_encode(body)]

        self.assertTrue(('foo[][dob][month]', 1) in values)
        self.assertTrue(('foo[][name]', 'bat') in values)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 're_123'


class RefundTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Refund.list()
        self.assert_requested(
            'get',
            '/v1/refunds'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Refund)

    def test_is_retrievable(self):
        resource = stripe.Refund.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/refunds/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Refund)

    def test_is_creatable(self):
        resource = stripe.Refund.create(
            charge='ch_123'
        )
        self.assert_requested(
            'post',
            '/v1/refunds'
        )
        self.assertIsInstance(resource, stripe.Refund)

    def test_is_saveable(self):
        resource = stripe.Refund.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/refunds/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Refund.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/refunds/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Refund)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'apwc_123'


class ApplePayDomainTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.ApplePayDomain.list()
        self.assert_requested(
            'get',
            '/v1/apple_pay/domains'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.ApplePayDomain)

    def test_is_retrievable(self):
        resource = stripe.ApplePayDomain.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/apple_pay/domains/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.ApplePayDomain)

    def test_is_creatable(self):
        resource = stripe.ApplePayDomain.create(
            domain_name='test.com',
        )
        self.assert_requested(
            'post',
            '/v1/apple_pay/domains'
        )
        self.assertIsInstance(resource, stripe.ApplePayDomain)

    def test_is_deletable(self):
        resource = stripe.ApplePayDomain.retrieve(TEST_RESOURCE_ID)
        # Unfortunately stripe-mock will return a resource with a different
        # ID, so we need to store the original ID for the request assertion
        resource_id = resource.id
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/apple_pay/domains/%s' % resource_id
        )
        self.assertTrue(resource.deleted)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'fee_123'
TEST_FEEREFUND_ID = 'fr_123'


class ApplicationFeeTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.ApplicationFee.list()
        self.assert_requested(
            'get',
            '/v1/application_fees'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.ApplicationFee)


class ApplicationFeeRefundsTests(StripeTestCase):
    def test_can_call_refund(self):
        appfee = stripe.ApplicationFee.retrieve(TEST_RESOURCE_ID)
        resource = appfee.refund()
        self.assert_requested(
            'post',
            '/v1/application_fees/%s/refund' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.ApplicationFee)
        self.assertTrue(resource is appfee)

    def test_is_listable(self):
        resources = stripe.ApplicationFee.list_refunds(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/application_fees/%s/refunds' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.ApplicationFeeRefund)

    def test_is_retrievable(self):
        resource = stripe.ApplicationFee.retrieve_refund(
            TEST_RESOURCE_ID,
            TEST_FEEREFUND_ID
        )
        self.assert_requested(
            'get',
            '/v1/application_fees/%s/refunds/%s' % (TEST_RESOURCE_ID,
                                                    TEST_FEEREFUND_ID)
        )
        self.assertIsInstance(resource, stripe.ApplicationFeeRefund)

    def test_is_creatable(self):
        resource = stripe.ApplicationFee.create_refund(
            TEST_RESOURCE_ID,
            amount=100
        )
        self.assert_requested(
            'post',
            '/v1/application_fees/%s/refunds' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.ApplicationFeeRefund)

    def test_is_modifiable(self):
        resource = stripe.ApplicationFee.modify_refund(
            TEST_RESOURCE_ID,
            TEST_FEEREFUND_ID,
            metadata={'foo': 'bar'}
        )
        self.assert_requested(
            'post',
            '/v1/application_fees/%s/refunds/%s' % (TEST_RESOURCE_ID,
                                                    TEST_FEEREFUND_ID)
        )
        self.assertIsInstance(resource, stripe.ApplicationFeeRefund)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'fr_123'
TEST_APPFEE_ID = 'fee_123'


class ApplicationFeeRefundTest(StripeTestCase):
    def test_is_saveable(self):
        appfee = stripe.ApplicationFee.retrieve(TEST_APPFEE_ID)
        resource = appfee.refunds.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/application_fees/%s/refunds/%s' % (resource.fee,
                                                    resource.id)
        )

    def test_is_modifiable(self):
        resource = stripe.ApplicationFeeRefund.modify(
            TEST_APPFEE_ID,
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/application_fees/%s/refunds/%s' % (TEST_APPFEE_ID,
                                                    TEST_RESOURCE_ID)
        )
        self.assertIsInstance(resource, stripe.ApplicationFeeRefund)

    def test_is_not_retrievable(self):
        with self.assertRaises(NotImplementedError):
            stripe.ApplicationFeeRefund.retrieve(TEST_RESOURCE_ID)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'tdsrc_123'


class ThreeDSecureTest(StripeTestCase):
    def test_is_retrievable(self):
        resource = stripe.ThreeDSecure.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/3d_secure/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.ThreeDSecure)

    def test_is_creatable(self):
        resource = stripe.ThreeDSecure.create(
            card="tok_123",
            amount=100,
            currency="usd",
            return_url="url"
        )
        self.assert_requested(
            'post',
            '/v1/3d_secure'
        )
        self.assertIsInstance(resource, stripe.ThreeDSecure)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'si_123'


class SubscriptionItemTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.SubscriptionItem.list()
        self.assert_requested(
            'get',
            '/v1/subscription_items'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.SubscriptionItem)

    def test_is_retrievable(self):
        resource = stripe.SubscriptionItem.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/subscription_items/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.SubscriptionItem)

    def test_is_creatable(self):
        resource = stripe.SubscriptionItem.create(
            plan='plan',
            subscription='sub_123'
        )
        self.assert_requested(
            'post',
            '/v1/subscription_items'
        )
        self.assertIsInstance(resource, stripe.SubscriptionItem)

    # TODO: Fix this test
    # def test_is_saveable(self):
    #    resource = stripe.SubscriptionItem.retrieve(TEST_RESOURCE_ID)
    #    resource.plan = 'plan'
    #    resource.save()
    #    self.assert_requested(
    #        'post',
    #        '/v1/subscription_items/%s' % resource.id,
    #        {
    #            'plan': 'plan',
    #        },
    #    )

    def test_is_modifiable(self):
        resource = stripe.SubscriptionItem.modify(
            TEST_RESOURCE_ID,
            plan='plan'
        )
        self.assert_requested(
            'post',
            '/v1/subscription_items/%s' % TEST_RESOURCE_ID,
            {
                'plan': 'plan',
            },
        )
        self.assertIsInstance(resource, stripe.SubscriptionItem)

    def test_is_deletable(self):
        resource = stripe.SubscriptionItem.retrieve(TEST_RESOURCE_ID)
        # Unfortunately stripe-mock will return a resource with a different
        # ID, so we need to store the original ID for the request assertion
        resource_id = resource.id
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/subscription_items/%s' % resource_id
        )
        self.assertTrue(resource.deleted)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'tr_123'
TEST_REVERSAL_ID = 'trr_123'


class TransferTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Transfer.list()
        self.assert_requested(
            'get',
            '/v1/transfers'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Transfer)

    def test_is_retrievable(self):
        resource = stripe.Transfer.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/transfers/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Transfer)

    def test_is_creatable(self):
        resource = stripe.Transfer.create(
            amount=100,
            currency='usd',
            destination='acct_123'
        )
        self.assert_requested(
            'post',
            '/v1/transfers'
        )
        self.assertIsInstance(resource, stripe.Transfer)

    def test_is_saveable(self):
        resource = stripe.Transfer.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/transfers/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Transfer.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/transfers/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Transfer)

    def test_is_cancelable(self):
        # stripe-mock does not handle this anymore as it was on an old
        # API version so we stub instead.
        self.stub_request(
            'post',
            '/v1/transfers/%s/cancel' % TEST_RESOURCE_ID,
            {
                'id': '%s' % TEST_RESOURCE_ID,
                'object': 'transfer',
                'status': 'canceled'
            }
        )
        transfer = stripe.Transfer.construct_from({
            'id': '%s' % TEST_RESOURCE_ID,
            'object': 'transfer'
        }, stripe.api_key)
        transfer_canceled = transfer.cancel()
        self.assert_requested(
            'post',
            '/v1/transfers/%s/cancel' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(transfer_canceled, stripe.Transfer)


class TransferReversalTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Transfer.list_reversals(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/transfers/%s/reversals' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Reversal)

    def test_is_retrievable(self):
        resource = stripe.Transfer.retrieve_reversal(
            TEST_RESOURCE_ID,
            TEST_REVERSAL_ID
        )
        self.assert_requested(
            'get',
            '/v1/transfers/%s/reversals/%s' % (TEST_RESOURCE_ID,
                                               TEST_REVERSAL_ID)
        )
        self.assertIsInstance(resource, stripe.Reversal)

    def test_is_creatable(self):
        resource = stripe.Transfer.create_reversal(
            TEST_RESOURCE_ID,
            amount=100
        )
        self.assert_requested(
            'post',
            '/v1/transfers/%s/reversals' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Reversal)

    def test_is_modifiable(self):
        resource = stripe.Transfer.modify_reversal(
            TEST_RESOURCE_ID,
            TEST_REVERSAL_ID,
            metadata={'foo': 'bar'}
        )
        self.assert_requested(
            'post',
            '/v1/transfers/%s/reversals/%s' % (TEST_RESOURCE_ID,
                                               TEST_REVERSAL_ID)
        )
        self.assertIsInstance(resource, stripe.Reversal)



from __future__ import absolute_import, division, print_function

from stripe.api_resources import abstract


class CountrySpec(abstract.ListableAPIResource):
    OBJECT_NAME = 'country_spec'

    @classmethod
    def class_name(cls):
        return 'country_spec'



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class SourceTransactionTest(StripeTestCase):
    def test_is_listable(self):
        source = stripe.Source.construct_from({
            'id': 'src_123',
            'object': 'source'
        }, stripe.api_key)
        source_transactions = source.source_transactions()
        self.assert_requested(
            'get',
            '/v1/sources/src_123/source_transactions'
        )
        self.assertIsInstance(source_transactions.data, list)
        self.assertIsInstance(source_transactions.data[0],
                              stripe.SourceTransaction)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import (StripeTestCase)


TEST_RESOURCE_ID = 'card_123'


class CardTest(StripeTestCase):
    def construct_resource(self, **params):
        card_dict = {
            'id': TEST_RESOURCE_ID,
            'object': 'card',
            'metadata': {},
        }
        card_dict.update(params)
        return stripe.Card.construct_from(card_dict, stripe.api_key)

    def test_has_account_instance_url(self):
        resource = self.construct_resource(account='acct_123')
        self.assertEqual(
            '/v1/accounts/acct_123/external_accounts/%s' % TEST_RESOURCE_ID,
            resource.instance_url()
        )

    def test_has_customer_instance_url(self):
        resource = self.construct_resource(customer='cus_123')
        self.assertEqual(
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID,
            resource.instance_url()
        )

    def test_has_recipient_instance_url(self):
        resource = self.construct_resource(recipient='rp_123')
        self.assertEqual(
            '/v1/recipients/rp_123/cards/%s' % TEST_RESOURCE_ID,
            resource.instance_url()
        )

    # The previous tests already ensure that the request will be routed to the
    # correct URL, so we only test the API operations once.

    def test_is_not_retrievable(self):
        with self.assertRaises(NotImplementedError):
            stripe.Card.retrieve(TEST_RESOURCE_ID)

    def test_is_saveable(self):
        resource = self.construct_resource(customer='cus_123')
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID
        )

    def test_is_not_modifiable(self):
        with self.assertRaises(NotImplementedError):
            stripe.Card.modify(
                TEST_RESOURCE_ID,
                metadata={'key': 'value'}
            )

    def test_is_deletable(self):
        resource = self.construct_resource(customer='cus_123')
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'txn_123'


class BalanceTransactionTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.BalanceTransaction.list()
        self.assert_requested(
            'get',
            '/v1/balance/history',
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.BalanceTransaction)



from __future__ import absolute_import, division, print_function

import warnings

import stripe
from tests.helper import (StripeTestCase)


TEST_RESOURCE_ID = 'cus_123'
TEST_SUB_ID = 'sub_123'
TEST_SOURCE_ID = 'ba_123'


class CustomerTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Customer.list()
        self.assert_requested(
            'get',
            '/v1/customers'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Customer)

    def test_is_retrievable(self):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/customers/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Customer)

    def test_is_creatable(self):
        resource = stripe.Customer.create()
        self.assert_requested(
            'post',
            '/v1/customers'
        )
        self.assertIsInstance(resource, stripe.Customer)

    def test_is_saveable(self):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/customers/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Customer.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/customers/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Customer)

    def test_is_deletable(self):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        # Unfortunately stripe-mock will return a resource with a different
        # ID, so we need to store the original ID for the request assertion
        resource_id = resource.id
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/customers/%s' % resource_id
        )
        self.assertTrue(resource.deleted)


# stripe-mock does not handle the legacy subscription endpoint so we stub
class CustomerLegacySubscriptionTest(StripeTestCase):
    def construct_resource(self):
        res_dict = {
            'id': TEST_RESOURCE_ID,
            'object': 'customer',
            'metadata': {},
        }
        return stripe.Customer.construct_from(res_dict, stripe.api_key)

    def test_can_update_legacy_subscription(self):
        self.stub_request(
            'post',
            '/v1/customers/%s/subscription' % TEST_RESOURCE_ID,
        )
        resource = self.construct_resource()
        with warnings.catch_warnings():
            warnings.simplefilter('ignore')
            resource.update_subscription(plan='plan')
        self.assert_requested(
            'post',
            '/v1/customers/%s/subscription' % TEST_RESOURCE_ID
        )

    def test_can_delete_legacy_subscription(self):
        self.stub_request(
            'delete',
            '/v1/customers/%s/subscription' % TEST_RESOURCE_ID
        )
        resource = self.construct_resource()
        with warnings.catch_warnings():
            warnings.simplefilter('ignore')
            resource.cancel_subscription()
        self.assert_requested(
            'delete',
            '/v1/customers/%s/subscription' % TEST_RESOURCE_ID
        )


class CustomerSourcesTests(StripeTestCase):
    def test_is_creatable(self):
        stripe.Customer.create_source(
            TEST_RESOURCE_ID,
            source='btok_123'
        )
        self.assert_requested(
            'post',
            '/v1/customers/%s/sources' % TEST_RESOURCE_ID
        )

    def test_is_retrievable(self):
        stripe.Customer.retrieve_source(
            TEST_RESOURCE_ID,
            TEST_SOURCE_ID
        )
        self.assert_requested(
            'get',
            '/v1/customers/%s/sources/%s' % (TEST_RESOURCE_ID,
                                             TEST_SOURCE_ID)
        )

    def test_is_modifiable(self):
        stripe.Customer.modify_source(
            TEST_RESOURCE_ID,
            TEST_SOURCE_ID,
            metadata={'foo': 'bar'}
        )
        self.assert_requested(
            'post',
            '/v1/customers/%s/sources/%s' % (TEST_RESOURCE_ID,
                                             TEST_SOURCE_ID)
        )

    def test_is_deletable(self):
        stripe.Customer.delete_source(
            TEST_RESOURCE_ID,
            TEST_SOURCE_ID
        )
        self.assert_requested(
            'delete',
            '/v1/customers/%s/sources/%s' % (TEST_RESOURCE_ID,
                                             TEST_SOURCE_ID)
        )

    def test_is_listable(self):
        resources = stripe.Customer.list_sources(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/customers/%s/sources' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resources.data, list)


class CustomerMethodsTests(StripeTestCase):
    def test_can_add_invoice_item(self):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.add_invoice_item(
            amount=100,
            currency='usd'
        )
        self.assert_requested(
            'post',
            '/v1/invoiceitems',
            {
                'amount': 100,
                'currency': 'usd',
                'customer': '%s' % resource.id
            }
        )

    def test_can_invoices(self):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.invoices()
        self.assert_requested(
            'get',
            '/v1/invoices',
            {
                'customer': '%s' % resource.id
            }
        )

    def test_can_invoice_items(self):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.invoice_items()
        self.assert_requested(
            'get',
            '/v1/invoiceitems',
            {
                'customer': '%s' % resource.id
            }
        )

    def test_can_charges(self):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.charges()
        self.assert_requested(
            'get',
            '/v1/charges',
            {
                'customer': '%s' % resource.id
            }
        )

    def test_can_delete_discount(self):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.delete_discount()
        self.assert_requested(
            'delete',
            '/v1/customers/%s/discount' % resource.id
        )



from __future__ import absolute_import, division, print_function

from six.moves.urllib.parse import parse_qs, urlparse

import stripe
from tests.helper import StripeTestCase


class OAuthTests(StripeTestCase):
    def test_authorize_url(self):
        url = stripe.OAuth.authorize_url(
            scope='read_write',
            state='csrf_token',
            stripe_user={
                'email': 'test@example.com',
                'url': 'https://example.com/profile/test',
                'country': 'US',
            })

        o = urlparse(url)
        params = parse_qs(o.query)

        self.assertEqual('https', o.scheme)
        self.assertEqual('connect.stripe.com', o.netloc)
        self.assertEqual('/oauth/authorize', o.path)

        self.assertEqual(['ca_123'], params['client_id'])
        self.assertEqual(['read_write'], params['scope'])
        self.assertEqual(['test@example.com'], params['stripe_user[email]'])
        self.assertEqual(
            ['https://example.com/profile/test'],
            params['stripe_user[url]']
        )
        self.assertEqual(['US'], params['stripe_user[country]'])

    def test_token(self):
        self.stub_request(
            'post',
            '/oauth/token',
            {
                'access_token': 'sk_access_token',
                'scope': 'read_only',
                'livemode': 'false',
                'token_type': 'bearer',
                'refresh_token': 'sk_refresh_token',
                'stripe_user_id': 'acct_test',
                'stripe_publishable_key': 'pk_test',
            }
        )

        resp = stripe.OAuth.token(
            grant_type='authorization_code',
            code='this_is_an_authorization_code',
        )
        self.assert_requested(
            'post',
            '/oauth/token',
            {
                'grant_type': 'authorization_code',
                'code': 'this_is_an_authorization_code',
            }
        )
        self.assertEqual('sk_access_token', resp['access_token'])

    def test_deauthorize(self):
        self.stub_request(
            'post',
            '/oauth/deauthorize',
            {
                'stripe_user_id': 'acct_test_deauth',
            }
        )

        resp = stripe.OAuth.deauthorize(stripe_user_id='acct_test_deauth')
        self.assert_requested(
            'post',
            '/oauth/deauthorize',
            {
                'client_id': 'ca_123',
                'stripe_user_id': 'acct_test_deauth',
            }
        )
        self.assertEqual('acct_test_deauth', resp['stripe_user_id'])



from __future__ import absolute_import, division, print_function

from stripe import util
from stripe.stripe_response import StripeResponse
from tests.helper import StripeTestCase


class StripeResponseTests(StripeTestCase):
    def test_idempotency_key(self):
        response, headers, body, code = \
            StripeResponseTests.mock_stripe_response()
        self.assertEqual(response.idempotency_key, headers['idempotency-key'])

    def test_request_id(self):
        response, headers, body, code = \
            StripeResponseTests.mock_stripe_response()
        self.assertEqual(response.request_id, headers['request-id'])

    def test_code(self):
        response, headers, body, code = \
            StripeResponseTests.mock_stripe_response()
        self.assertEqual(response.code, code)

    def test_headers(self):
        response, headers, body, code = \
            StripeResponseTests.mock_stripe_response()
        self.assertEqual(response.headers, headers)

    def test_body(self):
        response, headers, body, code = \
            StripeResponseTests.mock_stripe_response()
        self.assertEqual(response.body, body)

    def test_data(self):
        response, headers, body, code = \
            StripeResponseTests.mock_stripe_response()
        self.assertEqual(response.data, util.json.loads(body))

    @staticmethod
    def mock_stripe_response():
        code = 200
        headers = StripeResponseTests.mock_headers()
        body = StripeResponseTests.mock_body()
        response = StripeResponse(body, code, headers)
        return response, headers, body, code

    @staticmethod
    def mock_headers():
        return {
          'idempotency-key': '123456',
          'request-id': 'req_123456'
        }

    @staticmethod
    def mock_body():
        return '{ "id": "ch_12345", "object": "charge", "amount": 1 }'



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class MyResource(stripe.api_resources.abstract.APIResource):
    pass


class APIResourceTests(StripeTestCase):

    def test_retrieve_and_refresh(self):
        url = '/v1/myresources/foo%2A'
        self.stub_request(
            'get',
            url,
            {
                'id': 'foo2',
                'bobble': 'scrobble',
            },
            rheaders={'request-id': 'req_id'}
        )

        res = MyResource.retrieve('foo*', myparam=5)

        self.assert_requested(
            'get',
            url,
            {
                'myparam': 5,
            },
            None
        )
        self.assertEqual('scrobble', res.bobble)
        self.assertEqual('foo2', res.id)
        self.assertEqual('sk_test_123', res.api_key)

        self.assertTrue(res.last_response is not None)
        self.assertEqual('req_id', res.last_response.request_id)

        url = '/v1/myresources/foo2'
        self.stub_request(
            'get',
            url,
            {
                'frobble': 5,
            }
        )

        res = res.refresh()

        self.assert_requested(
            'get',
            url,
            {
                'myparam': 5,
            },
            None
        )
        self.assertEqual(5, res.frobble)
        self.assertRaises(KeyError, res.__getitem__, 'bobble')

    def test_convert_to_stripe_object(self):
        sample = {
            'foo': 'bar',
            'adict': {
                'object': 'charge',
                'id': 42,
                'amount': 7,
            },
            'alist': [
                {
                    'object': 'customer',
                    'name': 'chilango'
                }
            ]
        }

        converted = stripe.util.convert_to_stripe_object(
            sample, 'akey', None, None)

        # Types
        self.assertTrue(isinstance(converted,
                                   stripe.stripe_object.StripeObject))
        self.assertTrue(isinstance(converted.adict, stripe.Charge))
        self.assertEqual(1, len(converted.alist))
        self.assertTrue(isinstance(converted.alist[0], stripe.Customer))

        # Values
        self.assertEqual('bar', converted.foo)
        self.assertEqual(42, converted.adict.id)
        self.assertEqual('chilango', converted.alist[0].name)

        # Stripping
        # TODO: We should probably be stripping out this property
        # self.assertRaises(AttributeError, getattr, converted.adict, 'object')

    def test_convert_array_to_dict(self):
        out = stripe.util.convert_array_to_dict([{"foo": "bar"}])
        self.assertEqual({"0": {"foo": "bar"}}, out)
        self.assertEqual({"f": "b"},
                         stripe.util.convert_array_to_dict({"f": "b"}))

    def test_raise_on_incorrect_id_type(self):
        for obj in [None, 1, 3.14, dict(), list(), set(), tuple(), object()]:
            self.assertRaises(stripe.error.InvalidRequestError,
                              MyResource.retrieve, obj)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class MyUpdateable(stripe.api_resources.abstract.UpdateableAPIResource):
    pass


class UpdateableAPIResourceTests(StripeTestCase):
    def setUp(self):
        super(UpdateableAPIResourceTests, self).setUp()

        self.stub_request(
            'post',
            '/v1/myupdateables/myid',
            {
                'id': 'myid',
                'thats': 'it',
            },
            rheaders={'request-id': 'req_id'}
        )

        self.obj = MyUpdateable.construct_from({
            'id': 'myid',
            'foo': 'bar',
            'baz': 'boz',
            'metadata': {
                'size': 'l',
                'score': 4,
                'height': 10
            }
        }, 'mykey')

    def checkSave(self):
        self.assertTrue(self.obj is self.obj.save())

        self.assertEqual('it', self.obj.thats)
        # TODO: Should we force id to be retained?
        # self.assertEqual('myid', obj.id)
        self.assertRaises(AttributeError, getattr, self.obj, 'baz')

    def test_idempotent_save(self):
        self.obj.baz = 'updated'
        self.obj.save(idempotency_key='foo')

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'baz': 'updated',
            },
            {
                'Idempotency-Key': 'foo',
            },
        )

    def test_save(self):
        self.obj.baz = 'updated'
        self.obj.other = 'newval'
        self.obj.metadata.size = 'm'
        self.obj.metadata.info = 'a2'
        self.obj.metadata.height = None

        self.checkSave()

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'baz': 'updated',
                'other': 'newval',
                'metadata': {
                    'size': 'm',
                    'info': 'a2',
                    'height': '',
                }
            },
            None
        )

        self.assertTrue(self.obj.last_response is not None)
        self.assertEqual('req_id', self.obj.last_response.request_id)

        # Saving again should not cause any request.
        self.request_mock.reset_mock()
        self.checkSave()
        self.assert_no_request()

        # Setting the same value should cause a request.
        self.stub_request(
            'post',
            '/v1/myupdateables/myid',
            {
                'id': 'myid',
                'thats': 'it',
            }
        )

        self.obj.thats = 'it'
        self.checkSave()

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'thats': 'it',
            },
            None
        )

        # Changing the value should cause a request.
        self.stub_request(
            'post',
            '/v1/myupdateables/myid',
            {
                'id': 'myid',
                'thats': 'it',
            }
        )

        self.obj.id = 'myid'
        self.obj.thats = 'updated'
        self.checkSave()

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'thats': 'updated',
            },
            None
        )

    def test_add_key_to_nested_object(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'size': 'l',
                'score': 4,
                'height': 10
            }
        }, 'mykey')

        acct.legal_entity['first_name'] = 'bob'

        self.assertTrue(acct is acct.save())

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'first_name': 'bob',
                }
            },
            None
        )

    def test_save_nothing(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'metadata': {
                'key': 'value',
            }
        }, 'mykey')

        self.assertTrue(acct is acct.save())

        self.assert_no_request()

    def test_replace_nested_object(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'last_name': 'smith',
            }
        }, 'mykey')

        acct.legal_entity = {
            'first_name': 'bob',
        }

        self.assertTrue(acct is acct.save())

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'first_name': 'bob',
                    'last_name': '',
                }
            },
            None
        )

    def test_array_setting(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {}
        }, 'mykey')

        acct.legal_entity.additional_owners = [{'first_name': 'Bob'}]

        self.assertTrue(acct is acct.save())

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'additional_owners': [
                        {'first_name': 'Bob'}
                    ]
                }
            },
            None
        )

    def test_array_none(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'additional_owners': None,
            }
        }, 'mykey')

        acct.foo = 'bar'

        self.assertTrue(acct is acct.save())

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'foo': 'bar',
            },
            None
        )

    def test_array_insertion(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'additional_owners': []
            }
        }, 'mykey')

        acct.legal_entity.additional_owners.append({'first_name': 'Bob'})

        self.assertTrue(acct is acct.save())

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'additional_owners': {
                        '0': {'first_name': 'Bob'},
                    }
                }
            },
            None
        )

    def test_array_update(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'additional_owners': [
                    {'first_name': 'Bob'},
                    {'first_name': 'Jane'}
                ]
            }
        }, 'mykey')

        acct.legal_entity.additional_owners[1].first_name = 'Janet'

        self.assertTrue(acct is acct.save())

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {
                    'additional_owners': {
                        '0': {},
                        '1': {'first_name': 'Janet'}
                    }
                }
            },
            None
        )

    def test_array_noop(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'additional_owners': [{'first_name': 'Bob'}]
            },
            'currencies_supported': ['usd', 'cad']
        }, 'mykey')

        self.assertTrue(acct is acct.save())

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'legal_entity': {'additional_owners': {'0': {}}}
            },
            None
        )

    def test_hash_noop(self):
        acct = MyUpdateable.construct_from({
            'id': 'myid',
            'legal_entity': {
                'address': {'line1': '1 Two Three'}
            }
        }, 'mykey')

        self.assertTrue(acct is acct.save())

        self.assert_no_request()

    def test_save_replace_metadata_with_number(self):
        self.obj.baz = 'updated'
        self.obj.other = 'newval'
        self.obj.metadata = 3

        self.checkSave()

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'baz': 'updated',
                'other': 'newval',
                'metadata': 3,
            },
            None
        )

    def test_save_overwrite_metadata(self):
        self.obj.metadata = {}
        self.checkSave()

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'metadata': {
                    'size': '',
                    'score': '',
                    'height': '',
                }
            },
            None
        )

    def test_save_replace_metadata(self):
        self.obj.baz = 'updated'
        self.obj.other = 'newval'
        self.obj.metadata = {
            'size': 'm',
            'info': 'a2',
            'score': 4,
        }

        self.checkSave()

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'baz': 'updated',
                'other': 'newval',
                'metadata': {
                    'size': 'm',
                    'info': 'a2',
                    'height': '',
                    'score': 4,
                }
            },
            None
        )

    def test_save_update_metadata(self):
        self.obj.baz = 'updated'
        self.obj.other = 'newval'
        self.obj.metadata.update({
            'size': 'm',
            'info': 'a2',
            'score': 4,
        })

        self.checkSave()

        self.assert_requested(
            'post',
            '/v1/myupdateables/myid',
            {
                'baz': 'updated',
                'other': 'newval',
                'metadata': {
                    'size': 'm',
                    'info': 'a2',
                    'score': 4,
                }
            },
            None
        )

    def test_retrieve_and_update_with_stripe_version(self):
        self.stub_request(
            'get',
            '/v1/myupdateables/foo',
            {
                'id': 'foo',
                'bobble': 'scrobble',
            }
        )

        res = MyUpdateable.retrieve('foo', stripe_version='2017-08-15')

        self.request_mock.assert_api_version('2017-08-15')

        self.stub_request(
            'post',
            '/v1/myupdateables/foo',
            {
                'id': 'foo',
                'bobble': 'new_scrobble',
            }
        )

        res.bobble = 'new_scrobble'
        res.save()

        self.request_mock.assert_api_version('2017-08-15')



from __future__ import absolute_import, division, print_function

import warnings

import stripe
from tests.helper import StripeTestCase


class EphemeralKeyTest(StripeTestCase):

    def test_is_creatable(self):
        resource = stripe.EphemeralKey.create(
            customer='cus_123',
            stripe_version='2017-05-25'
        )
        self.request_mock.assert_api_version('2017-05-25')
        self.assert_requested(
            'post',
            '/v1/ephemeral_keys',
            {'customer': 'cus_123'}
        )
        self.assertIsInstance(resource, stripe.EphemeralKey)

    def test_raises_a_warning_when_using_api_version_arg(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter('always')

            resource = stripe.EphemeralKey.create(
                customer='cus_123',
                api_version='2017-05-25'
            )
            self.request_mock.assert_api_version('2017-05-25')
            self.assert_requested(
                'post',
                '/v1/ephemeral_keys',
                {'customer': 'cus_123'}
            )
            self.assertIsInstance(resource, stripe.EphemeralKey)

            self.assertEqual(1, len(w))
            self.assertEqual(w[0].category, DeprecationWarning)

    def test_is_not_creatable_without_an_explicit_api_version(self):
        with self.assertRaisesRegex(ValueError,
                                    'stripe_version must be specified'):
            stripe.EphemeralKey.create(customer='cus_123')

    def test_is_deletable(self):
        resource = stripe.EphemeralKey.create(
            customer='cus_123',
            stripe_version='2017-05-25'
        )
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/ephemeral_keys/%s' % resource.id
        )



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'btcrcv_123'


class BitcoinReceiverTest(StripeTestCase):
    def construct_resource(self, **params):
        res_dict = {
            'id': TEST_RESOURCE_ID,
            'object': 'bitcoin_receiver',
            'metadata': {},
        }
        res_dict.update(params)
        return stripe.BitcoinReceiver.construct_from(res_dict, stripe.api_key)

    def test_has_customer_instance_url(self):
        resource = self.construct_resource(customer='cus_123')
        self.assertEqual(
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID,
            resource.instance_url()
        )

    def test_has_receiver_instance_url(self):
        resource = self.construct_resource()
        self.assertEqual(
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID,
            resource.instance_url()
        )

    def test_is_listable(self):
        resources = stripe.BitcoinReceiver.list()
        self.assert_requested(
            'get',
            '/v1/bitcoin/receivers'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.BitcoinReceiver)

    def test_is_retrievable(self):
        resource = stripe.BitcoinReceiver.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.BitcoinReceiver)

    # stripe-mock does not handle most write operations anymore so we stub
    # each one instead. This endpoint/resource is mostly deprecated today.
    # The previous tests already ensure that the request will be routed to the
    # correct URL, so we also only test the API operations once.

    def test_is_creatable(self):
        self.stub_request(
            'post',
            '/v1/bitcoin/receivers',
            {
                'id': '%s' % TEST_RESOURCE_ID,
                'object': 'bitcoin_receiver'
            }
        )
        resource = stripe.BitcoinReceiver.create()
        self.assert_requested(
            'post',
            '/v1/bitcoin/receivers'
        )
        self.assertIsInstance(resource, stripe.BitcoinReceiver)

    def test_is_saveable(self):
        self.stub_request(
            'post',
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID
        )
        resource = self.construct_resource()
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID
        )

    def test_is_modifiable(self):
        self.stub_request(
            'post',
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID
        )
        stripe.BitcoinReceiver.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID
        )

    def test_is_deletable(self):
        self.stub_request(
            'delete',
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID
        )
        resource = self.construct_resource()
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/bitcoin/receivers/%s' % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_SUBSCRIPTION_ITEM_ID = 'si_123'


class UsageRecordTest(StripeTestCase):
    def test_is_creatable(self):
        resource = stripe.UsageRecord.create(
            subscription_item=TEST_SUBSCRIPTION_ITEM_ID,
            quantity=5000,
            timestamp=1524182400,
            action='increment',
        )
        self.assert_requested(
            'post',
            '/v1/subscription_items/%s/usage_records' % (
                TEST_SUBSCRIPTION_ITEM_ID
            )
        )
        self.assertIsInstance(resource, stripe.UsageRecord)

    def test_raises_when_creating_without_subscription_item(self):
        with self.assertRaises(ValueError):
            stripe.UsageRecord.create(
                quantity=5000,
                timestamp=1524182400,
                action='increment',
            )



from __future__ import absolute_import, division, print_function

import time

import stripe
from stripe import six
from tests.helper import StripeTestCase


DUMMY_WEBHOOK_PAYLOAD = """{
  "id": "evt_test_webhook",
  "object": "event"
}"""

DUMMY_WEBHOOK_SECRET = 'whsec_test_secret'


class WebhookTests(StripeTestCase):
    def test_construct_event(self):
        header = WebhookSignatureTests.generate_header()
        event = stripe.Webhook.construct_event(
            DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET)
        self.assertTrue(isinstance(event, stripe.Event))

    def test_raise_on_json_error(self):
        payload = 'this is not valid JSON'
        header = WebhookSignatureTests.generate_header(payload=payload)
        with self.assertRaises(ValueError):
            stripe.Webhook.construct_event(
                payload, header, DUMMY_WEBHOOK_SECRET)

    def test_raise_on_invalid_header(self):
        header = 'bad_header'
        with self.assertRaises(stripe.error.SignatureVerificationError):
            stripe.Webhook.construct_event(
                DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET)

    def test_construct_event_from_bytearray(self):
        header = WebhookSignatureTests.generate_header()
        payload = bytearray(DUMMY_WEBHOOK_PAYLOAD, 'utf-8')
        event = stripe.Webhook.construct_event(
            payload, header, DUMMY_WEBHOOK_SECRET)
        self.assertTrue(isinstance(event, stripe.Event))

    def test_construct_event_from_bytes(self):
        # This test is only applicable to Python 3 as `bytes` is not a symbol
        # in Python 2.
        if six.PY2:
            return

        header = WebhookSignatureTests.generate_header()
        payload = bytes(DUMMY_WEBHOOK_PAYLOAD, 'utf-8')
        event = stripe.Webhook.construct_event(
            payload, header, DUMMY_WEBHOOK_SECRET)
        self.assertTrue(isinstance(event, stripe.Event))


class WebhookSignatureTests(StripeTestCase):
    @staticmethod
    def generate_header(**kwargs):
        timestamp = kwargs.get('timestamp', int(time.time()))
        payload = kwargs.get('payload', DUMMY_WEBHOOK_PAYLOAD)
        secret = kwargs.get('secret', DUMMY_WEBHOOK_SECRET)
        scheme = kwargs.get('scheme', stripe.WebhookSignature.EXPECTED_SCHEME)
        signature = kwargs.get('signature', None)
        if signature is None:
            payload_to_sign = "%d.%s" % (timestamp, payload)
            signature = stripe.WebhookSignature._compute_signature(
                payload_to_sign, secret)
        header = "t=%d,%s=%s" % (timestamp, scheme, signature)
        return header

    def test_raise_on_malformed_header(self):
        header = "i'm not even a real signature header"
        with self.assertRaisesRegex(
                stripe.error.SignatureVerificationError,
                "Unable to extract timestamp and signatures from header"):
            stripe.WebhookSignature.verify_header(
                DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET)

    def test_raise_on_no_signatures_with_expected_scheme(self):
        header = self.generate_header(scheme='v0')
        with self.assertRaisesRegex(
                stripe.error.SignatureVerificationError,
                "No signatures found with expected scheme v1"):
            stripe.WebhookSignature.verify_header(
                DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET)

    def test_raise_on_no_valid_signatures_for_payload(self):
        header = self.generate_header(signature='bad_signature')
        with self.assertRaisesRegex(
                stripe.error.SignatureVerificationError,
                "No signatures found matching the expected signature for "
                "payload"):
            stripe.WebhookSignature.verify_header(
                DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET)

    def test_raise_on_timestamp_outside_tolerance(self):
        header = self.generate_header(timestamp=int(time.time()) - 15)
        with self.assertRaisesRegex(
                stripe.error.SignatureVerificationError,
                "Timestamp outside the tolerance zone"):
            stripe.WebhookSignature.verify_header(
                DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET,
                tolerance=10)

    def test_valid_header_and_signature(self):
        header = self.generate_header()
        self.assertTrue(stripe.WebhookSignature.verify_header(
            DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET,
            tolerance=10))

    def test_header_contains_valid_signature(self):
        header = self.generate_header() + ",v1=bad_signature"
        self.assertTrue(stripe.WebhookSignature.verify_header(
            DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET,
            tolerance=10))

    def test_timestamp_off_but_no_tolerance(self):
        header = self.generate_header(timestamp=12345)
        self.assertTrue(stripe.WebhookSignature.verify_header(
            DUMMY_WEBHOOK_PAYLOAD, header, DUMMY_WEBHOOK_SECRET))



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'sub_123'


class SubscriptionTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Subscription.list()
        self.assert_requested(
            'get',
            '/v1/subscriptions'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Subscription)

    def test_is_retrievable(self):
        resource = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/subscriptions/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Subscription)

    def test_is_creatable(self):
        resource = stripe.Subscription.create(
            customer='cus_123',
        )
        self.assert_requested(
            'post',
            '/v1/subscriptions'
        )
        self.assertIsInstance(resource, stripe.Subscription)

    def test_is_saveable(self):
        resource = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/subscriptions/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Subscription.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/subscriptions/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Subscription)

    def test_is_deletable(self):
        resource = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/subscriptions/%s' % resource.id
        )
        self.assertIsInstance(resource, stripe.Subscription)

    def test_can_delete_discount(self):
        sub = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
        sub.delete_discount()
        self.assert_requested(
            'delete',
            '/v1/subscriptions/%s/discount' % sub.id
        )

    # Test create/modify methods with subscription items

    def test_is_creatable_with_items(self):
        resource = stripe.Subscription.create(
            customer='cus_123',
            items=[{"plan": "foo", "quantity": 3}]
        )
        self.assert_requested(
            'post',
            '/v1/subscriptions',
            {
                'customer': 'cus_123',
                'items': {
                    "0": {
                        "plan": "foo",
                        "quantity": 3
                    },
                },
            },
        )
        self.assertIsInstance(resource, stripe.Subscription)

    def test_is_modifiable_with_items(self):
        resource = stripe.Subscription.modify(
            TEST_RESOURCE_ID,
            items=[{"id": "si", "plan": "foo"}]
        )
        self.assert_requested(
            'post',
            '/v1/subscriptions/%s' % TEST_RESOURCE_ID,
            {
                'items': {
                    "0": {
                        "plan": "foo",
                        "id": "si"
                    },
                },
            },
        )
        self.assertIsInstance(resource, stripe.Subscription)

    # TODO: Fix this test
    # def test_is_saveable_with_items(self):
    #    resource = stripe.Subscription.retrieve(TEST_RESOURCE_ID)
    #    resource.items = [{"id": "si", "plan": "foo"}]
    #    resource.save()
    #    self.assert_requested(
    #        'post',
    #        '/v1/subscriptions/%s' % TEST_RESOURCE_ID,
    #        {
    #            'items': {
    #                "0": {
    #                    "plan": "foo",
    #                    "id": "si"
    #                },
    #            },
    #        },
    #    )
    #    self.assertIsInstance(resource, stripe.Subscription)

    # TODO: Test serialize



from __future__ import absolute_import, division, print_function

import warnings

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'src_123'


class SourceTest(StripeTestCase):

    def test_is_retrievable(self):
        resource = stripe.Source.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/sources/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Source)

    def test_is_creatable(self):
        resource = stripe.Source.create(
            type='card',
            token='tok_123'
        )
        self.assert_requested(
            'post',
            '/v1/sources'
        )
        self.assertIsInstance(resource, stripe.Source)

    def test_is_saveable(self):
        resource = stripe.Source.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/sources/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Source.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/sources/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Source)

    def test_is_detachable_when_attached(self):
        resource = stripe.Source.construct_from({
            'id': TEST_RESOURCE_ID,
            'object': 'source',
            'customer': 'cus_123'
        }, stripe.api_key)
        source = resource.detach()
        self.assertTrue(source is resource)
        self.assert_requested(
            'delete',
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID
        )

    def test_is_not_detachable_when_unattached(self):
        resource = stripe.Source.retrieve(TEST_RESOURCE_ID)
        self.assertRaises(NotImplementedError, resource.detach)

    def test_raises_a_warning_when_calling_delete(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter('always')

            resource = stripe.Source.construct_from({
                'id': TEST_RESOURCE_ID,
                'object': 'source',
                'customer': 'cus_123'
            }, stripe.api_key)
            resource.delete()

            self.assertEqual(1, len(w))
            self.assertEqual(w[0].category, DeprecationWarning)

    def test_is_verifiable(self):
        resource = stripe.Source.retrieve(TEST_RESOURCE_ID)
        source = resource.verify(values=[1, 2])
        self.assertTrue(source is resource)
        self.assert_requested(
            'post',
            '/v1/sources/%s/verify' % TEST_RESOURCE_ID,
            {'values': [1, 2]}
        )



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'acct_123'
TEST_EXTERNALACCOUNT_ID = 'ba_123'


class AccountTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Account.list()
        self.assert_requested(
            'get',
            '/v1/accounts'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Account)

    def test_is_retrievable(self):
        resource = stripe.Account.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/accounts/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Account)

    def test_is_creatable(self):
        resource = stripe.Account.create(
            country='US',
            type='custom'
        )
        self.assert_requested(
            'post',
            '/v1/accounts'
        )
        self.assertIsInstance(resource, stripe.Account)

    def test_is_saveable(self):
        account = stripe.Account.retrieve(TEST_RESOURCE_ID)
        account.metadata['key'] = 'value'
        resource = account.save()
        self.assert_requested(
            'post',
            '/v1/accounts/%s' % resource.id
        )
        self.assertIsInstance(resource, stripe.Account)
        self.assertTrue(resource is account)

    def test_is_saveable_with_additional_owners(self):
        # stripe-mock does not return additional owners so we construct
        account = stripe.Account.construct_from({
            'id': '%s' % TEST_RESOURCE_ID,
            'legal_entity': {
                'additional_owners': [{
                    'first_name': 'name',
                    'verification': {},
                }],
            }
        }, stripe.api_key)
        owner = account.legal_entity.additional_owners[0]
        owner.verification.document = 'file_foo'
        resource = account.save()
        self.assert_requested(
            'post',
            '/v1/accounts/%s' % TEST_RESOURCE_ID,
            {
                'legal_entity': {
                    'additional_owners': {
                        '0': {
                            'verification': {
                                'document': 'file_foo',
                            },
                        },
                    },
                },
            }
        )
        self.assertIsInstance(resource, stripe.Account)
        self.assertTrue(resource is account)

    def test_is_modifiable(self):
        resource = stripe.Account.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/accounts/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Account)

    def test_is_deletable(self):
        resource = stripe.Account.retrieve(TEST_RESOURCE_ID)
        # Unfortunately stripe-mock will return a resource with a different
        # ID, so we need to store the original ID for the request assertion
        resource_id = resource.id
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/accounts/%s' % resource_id
        )
        self.assertTrue(resource.deleted)

    def test_can_retrieve_no_id(self):
        resource = stripe.Account.retrieve()
        self.assert_requested(
            'get',
            '/v1/account'
        )
        self.assertIsInstance(resource, stripe.Account)

    def test_can_reject(self):
        account = stripe.Account.retrieve(TEST_RESOURCE_ID)
        resource = account.reject(reason='fraud')
        self.assert_requested(
            'post',
            '/v1/accounts/%s/reject' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Account)
        self.assertTrue(resource is account)

    def test_is_deauthorizable(self):
        account = stripe.Account.retrieve(TEST_RESOURCE_ID)
        self.stub_request(
            'post',
            '/oauth/deauthorize',
            {
                'stripe_user_id': account.id,
            }
        )
        account.deauthorize()
        self.assert_requested(
           'post',
           '/oauth/deauthorize',
           {
               'client_id': stripe.client_id,
               'stripe_user_id': account.id,
           }
        )


class AccountExternalAccountsTests(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Account.list_external_accounts(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/accounts/%s/external_accounts' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Card)

    def test_is_retrievable(self):
        resource = stripe.Account.retrieve_external_account(
            TEST_RESOURCE_ID,
            TEST_EXTERNALACCOUNT_ID
        )
        self.assert_requested(
            'get',
            '/v1/accounts/%s/external_accounts/%s' % (TEST_RESOURCE_ID,
                                                      TEST_EXTERNALACCOUNT_ID)
        )
        self.assertIsInstance(resource, stripe.BankAccount)

    def test_is_creatable(self):
        resource = stripe.Account.create_external_account(
            TEST_RESOURCE_ID,
            external_account='btok_123'
        )
        self.assert_requested(
            'post',
            '/v1/accounts/%s/external_accounts' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.BankAccount)

    def test_is_modifiable(self):
        resource = stripe.Account.modify_external_account(
            TEST_RESOURCE_ID,
            TEST_EXTERNALACCOUNT_ID,
            metadata={'foo': 'bar'}
        )
        self.assert_requested(
            'post',
            '/v1/accounts/%s/external_accounts/%s' % (TEST_RESOURCE_ID,
                                                      TEST_EXTERNALACCOUNT_ID)
        )
        self.assertIsInstance(resource, stripe.BankAccount)

    def test_is_deletable(self):
        resource = stripe.Account.delete_external_account(
            TEST_RESOURCE_ID,
            TEST_EXTERNALACCOUNT_ID
        )
        self.assert_requested(
            'delete',
            '/v1/accounts/%s/external_accounts/%s' % (TEST_RESOURCE_ID,
                                                      TEST_EXTERNALACCOUNT_ID)
        )
        self.assertTrue(resource.deleted)


class AccountLoginLinksTests(StripeTestCase):
    def test_is_creatable(self):
        resource = stripe.Account.create_login_link(TEST_RESOURCE_ID)
        self.assert_requested(
            'post',
            '/v1/accounts/%s/login_links' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.LoginLink)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


class MyCreatable(stripe.api_resources.abstract.CreateableAPIResource):
    pass


class CreateableAPIResourceTests(StripeTestCase):

    def test_create(self):
        self.stub_request(
            'post',
            '/v1/mycreatables',
            {
                'object': 'charge',
                'foo': 'bar',
            },
            rheaders={'request-id': 'req_id'}
        )

        res = MyCreatable.create()

        self.assert_requested(
            'post',
            '/v1/mycreatables',
            {}
        )
        self.assertTrue(isinstance(res, stripe.Charge))
        self.assertEqual('bar', res.foo)

        self.assertTrue(res.last_response is not None)
        self.assertEqual('req_id', res.last_response.request_id)

    def test_idempotent_create(self):
        self.stub_request(
            'post',
            '/v1/mycreatables',
            {
                'object': 'charge',
                'foo': 'bar',
            },
            rheaders={'idempotency-key': 'foo'}
        )

        res = MyCreatable.create(idempotency_key='foo')

        self.assert_requested(
            'post',
            '/v1/mycreatables',
            {},
            {'Idempotency-Key': 'foo'}
        )
        self.assertTrue(isinstance(res, stripe.Charge))
        self.assertEqual('bar', res.foo)

        self.assertTrue(res.last_response is not None)
        self.assertEqual('foo', res.last_response.idempotency_key)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = '250FF'


class CouponTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Coupon.list()
        self.assert_requested(
            'get',
            '/v1/coupons'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Coupon)

    def test_is_retrievable(self):
        resource = stripe.Coupon.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/coupons/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Coupon)

    def test_is_creatable(self):
        resource = stripe.Coupon.create(
            percent_off=25,
            duration='repeating',
            duration_in_months=3,
            id='250FF'
        )
        self.assert_requested(
            'post',
            '/v1/coupons'
        )
        self.assertIsInstance(resource, stripe.Coupon)

    def test_is_saveable(self):
        resource = stripe.Coupon.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/coupons/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Coupon.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/coupons/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Coupon)

    def test_is_deletable(self):
        resource = stripe.Coupon.retrieve(TEST_RESOURCE_ID)
        # Unfortunately stripe-mock will return a resource with a different
        # ID, so we need to store the original ID for the request assertion
        resource_id = resource.id
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/coupons/%s' % resource_id
        )
        self.assertTrue(resource.deleted)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from stripe import CardError, StripeError, six
from tests.helper import StripeTestCase


class StripeErrorTests(StripeTestCase):

    def test_formatting(self):
        err = StripeError(u'öre')
        self.assertEqual(u'öre', six.text_type(err))
        if six.PY2:
            self.assertEqual('\xc3\xb6re', str(err))
        else:
            self.assertEqual(u'öre', str(err))

    def test_formatting_with_request_id(self):
        err = StripeError(u'öre', headers={'request-id': '123'})
        self.assertEqual(u'Request 123: öre', six.text_type(err))
        if six.PY2:
            self.assertEqual('Request 123: \xc3\xb6re', str(err))
        else:
            self.assertEqual(u'Request 123: öre', str(err))

    def test_formatting_with_message_none_and_request_id(self):
        err = StripeError(None, headers={'request-id': '123'})
        self.assertEqual(u'Request 123: <empty message>', six.text_type(err))
        if six.PY2:
            self.assertEqual('Request 123: <empty message>', str(err))
        else:
            self.assertEqual('Request 123: <empty message>', str(err))

    def test_formatting_with_message_none_and_request_id_none(self):
        err = StripeError(None)
        self.assertEqual(u'<empty message>', six.text_type(err))
        if six.PY2:
            self.assertEqual('<empty message>', str(err))
        else:
            self.assertEqual('<empty message>', str(err))

    def test_repr(self):
        err = StripeError(u'öre', headers={'request-id': '123'})
        if six.PY2:
            self.assertEquals(repr(err), (
                "StripeError(message=u'\\xf6re', http_status=None, "
                "request_id='123')"))
        else:
            self.assertEquals(repr(err), (
                "StripeError(message='öre', http_status=None, "
                "request_id='123')"))


class StripeErrorWithParamCodeTests(StripeTestCase):

    def test_repr(self):
        err = CardError(u'öre', param='cparam', code='ccode', http_status=403,
                        headers={'request-id': '123'})
        if six.PY2:
            self.assertEquals(repr(err), (
                "CardError(message=u'\\xf6re', param='cparam', code='ccode', "
                "http_status=403, request_id='123')"))
        else:
            self.assertEquals(repr(err), (
                "CardError(message='öre', param='cparam', code='ccode', "
                "http_status=403, request_id='123')"))



import os
import sys
import warnings

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


path, script = os.path.split(sys.argv[0])
os.chdir(os.path.abspath(path))


with open('LONG_DESCRIPTION.rst') as f:
    long_description = f.read()

version_contents = {}
with open(os.path.join('stripe', 'version.py')) as f:
    exec(f.read(), version_contents)

tests_require = [
    'mock',
    'unittest2',
]

setup(
    name='stripe',
    version=version_contents['VERSION'],
    description='Python bindings for the Stripe API',
    long_description=long_description,
    author='Stripe',
    author_email='support@stripe.com',
    url='https://github.com/stripe/stripe-python',
    license='MIT',
    packages=['stripe', 'stripe.api_resources',
              'stripe.api_resources.abstract'],
    package_data={'stripe': ['data/ca-certificates.crt']},
    install_requires=[
        'requests >= 0.8.8',
    ],
    test_suite='tests',
    tests_require=tests_require,
    extras_require={
        'testing': tests_require,
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ])



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import (StripeTestCase)


TEST_RESOURCE_ID = 'aliacc_123'


class AlipayAccountTest(StripeTestCase):
    def construct_resource(self):
        alipay_dict = {
            'id': TEST_RESOURCE_ID,
            'object': 'alipay_account',
            'metadata': {},
            'customer': 'cus_123'
        }
        return stripe.AlipayAccount.construct_from(alipay_dict, stripe.api_key)

    def test_has_instance_url(self):
        resource = self.construct_resource()
        self.assertEqual(
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID,
            resource.instance_url()
        )

    def test_is_modifiable(self):
        stripe.AlipayAccount.modify(
            'cus_123',
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID
        )

    def test_is_not_retrievable(self):
        with self.assertRaises(NotImplementedError):
            stripe.AlipayAccount.retrieve(TEST_RESOURCE_ID)

    # Below, we don't use stripe-mock as it always returns a Bank Account
    # object when you hit /v1/customers/%s/sources/%s

    def test_is_saveable(self):
        resource = self.construct_resource()
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID
        )

    def test_is_deletable(self):
        resource = self.construct_resource()
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID
        )



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'ch_123'


class ChargeTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Charge.list()
        self.assert_requested(
            'get',
            '/v1/charges'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Charge)

    def test_is_retrievable(self):
        resource = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/charges/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Charge)

    def test_is_creatable(self):
        resource = stripe.Charge.create(
            amount=100,
            currency='usd',
            source='tok_123'
        )
        self.assert_requested(
            'post',
            '/v1/charges'
        )
        self.assertIsInstance(resource, stripe.Charge)

    def test_is_saveable(self):
        resource = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/charges/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Charge.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/charges/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Charge)


class ChargeMethodsTest(StripeTestCase):
    def test_can_refund(self):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.refund()
        self.assert_requested(
            'post',
            '/v1/charges/%s/refund' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Charge)

    def test_can_capture(self):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.capture()
        self.assert_requested(
            'post',
            '/v1/charges/%s/capture' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Charge)

    def test_can_update_dispute(self):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.update_dispute()
        self.assert_requested(
            'post',
            '/v1/charges/%s/dispute' % charge.id
        )
        self.assertIsInstance(resource, stripe.Dispute)

    def test_can_close_dispute(self):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.close_dispute()
        self.assert_requested(
            'post',
            '/v1/charges/%s/dispute/close' % charge.id
        )
        self.assertIsInstance(resource, stripe.Dispute)

    def test_can_mark_as_fraudulent(self):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.mark_as_fraudulent()
        self.assert_requested(
            'post',
            '/v1/charges/%s' % charge.id,
            {
                'fraud_details': {'user_report': 'fraudulent'}
            }
        )
        self.assertIsInstance(resource, stripe.Charge)

    def test_can_mark_as_safe(self):
        charge = stripe.Charge.retrieve(TEST_RESOURCE_ID)
        resource = charge.mark_as_safe()
        self.assert_requested(
            'post',
            '/v1/charges/%s' % charge.id,
            {
                'fraud_details': {'user_report': 'safe'}
            }
        )
        self.assertIsInstance(resource, stripe.Charge)



import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'tu_123'


class TopupTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Topup.list()
        self.assert_requested(
            'get',
            '/v1/topups'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Topup)

    def test_is_retrievable(self):
        resource = stripe.Topup.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/topups/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Topup)

    def test_is_creatable(self):
        resource = stripe.Topup.create(
            amount=100,
            currency='usd',
            source='src_123',
            description='description',
            statement_descriptor='statement descriptor'
        )
        self.assert_requested(
            'post',
            '/v1/topups'
        )
        self.assertIsInstance(resource, stripe.Topup)

    def test_is_saveable(self):
        resource = stripe.Topup.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/topups/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Topup.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/topups/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Topup)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'or_123'


class OrderTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Order.list()
        self.assert_requested(
            'get',
            '/v1/orders'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Order)

    def test_is_retrievable(self):
        resource = stripe.Order.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/orders/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Order)

    def test_is_creatable(self):
        resource = stripe.Order.create(
            currency='usd'
        )
        self.assert_requested(
            'post',
            '/v1/orders'
        )
        self.assertIsInstance(resource, stripe.Order)

    def test_is_saveable(self):
        resource = stripe.Order.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/orders/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Order.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/orders/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Order)

    def test_is_payable(self):
        order = stripe.Order.retrieve(TEST_RESOURCE_ID)
        resource = order.pay(source='src_123')
        self.assert_requested(
            'post',
            '/v1/orders/%s/pay' % order.id,
            {
                'source': 'src_123'
            }
        )
        self.assertTrue(resource, stripe.Order)
        self.assertTrue(resource is order)

    def test_is_returnable(self):
        order = stripe.Order.retrieve(TEST_RESOURCE_ID)
        resource = order.return_order()
        self.assert_requested(
            'post',
            '/v1/orders/%s/returns' % order.id
        )
        self.assertIsInstance(resource, stripe.OrderReturn)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'in_123'


class InvoiceTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Invoice.list()
        self.assert_requested(
            'get',
            '/v1/invoices'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Invoice)

    def test_is_retrievable(self):
        resource = stripe.Invoice.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/invoices/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Invoice)

    def test_is_creatable(self):
        resource = stripe.Invoice.create(
            customer='cus_123'
        )
        self.assert_requested(
            'post',
            '/v1/invoices'
        )
        self.assertIsInstance(resource, stripe.Invoice)

    def test_is_saveable(self):
        resource = stripe.Invoice.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/invoices/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Invoice.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/invoices/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Invoice)

    def test_can_pay(self):
        resource = stripe.Invoice.retrieve(TEST_RESOURCE_ID)
        resource = resource.pay()
        self.assert_requested(
            'post',
            '/v1/invoices/%s/pay' % resource.id
        )
        self.assertIsInstance(resource, stripe.Invoice)

    def test_can_upcoming(self):
        resource = stripe.Invoice.upcoming()
        self.assert_requested(
            'get',
            '/v1/invoices/upcoming'
        )
        self.assertIsInstance(resource, stripe.Invoice)

    def test_can_upcoming_and_subscription_items(self):
        resource = stripe.Invoice.upcoming(
            subscription_items=[
                {"plan": "foo", "quantity": 3}
            ]
        )
        self.assert_requested(
            'get',
            '/v1/invoices/upcoming',
            {
                'subscription_items': {
                    "0": {
                        "plan": "foo",
                        "quantity": 3,
                    },
                },
            },
        )
        self.assertIsInstance(resource, stripe.Invoice)



from __future__ import absolute_import, division, print_function

import tempfile

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'file_123'


class FileUploadTest(StripeTestCase):
    FIXTURE = {
        'id': TEST_RESOURCE_ID,
        'object': 'file_upload'
    }

    def test_is_listable(self):
        self.stub_request(
            'get',
            '/v1/files',
            {
                'object': 'list',
                'data': [self.FIXTURE],
            }
        )

        resources = stripe.FileUpload.list()
        self.assert_requested(
            'get',
            '/v1/files'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.FileUpload)

    def test_is_retrievable(self):
        self.stub_request(
            'get',
            '/v1/files/%s' % TEST_RESOURCE_ID,
            self.FIXTURE
        )

        resource = stripe.FileUpload.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/files/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.FileUpload)

    def test_is_creatable(self):
        self.stub_request(
            'post',
            '/v1/files',
            self.FIXTURE
        )

        test_file = tempfile.TemporaryFile()
        resource = stripe.FileUpload.create(
            purpose='dispute_evidence',
            file=test_file
        )
        self.assert_requested(
            'post',
            '/v1/files',
            headers={'Content-Type': 'multipart/form-data'}
        )
        self.assertIsInstance(resource, stripe.FileUpload)
